import {AsyncPipe} from '@angular/common';
import {ChangeDetectorRef, Component, OnInit} from '@angular/core';
import {MatFabButton, MatMiniFabButton} from '@angular/material/button';
import {MatFormFieldModule} from '@angular/material/form-field';
import {MatIcon} from '@angular/material/icon';
import {MatInputModule} from '@angular/material/input';
import {MatSelectModule} from '@angular/material/select';
import {
    MatTableDataSource, MatTableModule
} from '@angular/material/table';
import {AuthService} from '@core/auth/auth.service';
import UserData from '@core/auth/UserData';
import {AdministratablePlayer, AdminService, PlayerFilter} from '@core/server';

@Component({
    selector: 'app-dashboard',
    imports: [
        AsyncPipe,
        MatIcon,
        MatInputModule,
        MatTableModule,
        MatFormFieldModule,
        MatMiniFabButton,
        MatFabButton,
        MatSelectModule
    ],
    templateUrl: './dashboard.component.html',
    styleUrl: './dashboard.component.scss'
})
export class DashboardComponent implements OnInit {
    displayedColumns: Array<string> = ['username', 'actions', 'banned'];
    selectedFilter: string = 'all';
    originalUsersDataSource: MatTableDataSource<AdministratablePlayer> = new MatTableDataSource<AdministratablePlayer>([]);
    filteredUsersDataSource: MatTableDataSource<AdministratablePlayer> = new MatTableDataSource<AdministratablePlayer>([]);

    constructor(private adminService: AdminService, protected authService: AuthService, private cdr: ChangeDetectorRef) {
    }

    ngOnInit(): void {
        this.authService.$user.subscribe((user) => {
            if (user != undefined) {
                return this.fetchPlayers('');
            }
        });
    }

    fetchPlayers(query: string): void {
        const filter: PlayerFilter = {
            page: 0,
            pageSize: 32,
            order: PlayerFilter.OrderEnum.Ascending,
            username: query,
            sortBy: 'username'
        };

        this.adminService.getAllPlayers(filter).subscribe(
            (players) => {
                this.originalUsersDataSource.data = players; // Store original data
                this.filteredUsersDataSource.data = [...players]; // Apply initial filter
            },
            (error) => {
                console.error('Error fetching players:', error);
            }
        );
    }

    banPlayer(user: UserData): void {
        this.adminService.banPlayer(user.id).subscribe(() => {
            user.banned = true;
            this.originalUsersDataSource.data = [...this.originalUsersDataSource.data];
            this.applyFilter(this.selectedFilter);
        });
    }

    unbanPlayer(user: UserData): void {
        this.adminService.unbanPlayer(user.id).subscribe(() => {
            user.banned = false;
            this.originalUsersDataSource.data = [...this.originalUsersDataSource.data];
            this.applyFilter(this.selectedFilter);
        });
    }

    onSearch(searchbar: HTMLInputElement): void {
        this.fetchPlayers(searchbar.value);
    }

    resetFilter(searchbar: HTMLInputElement): void {
        searchbar.value = '';
        this.fetchPlayers('');
    }

    applyFilter(filter: string): void {
        this.selectedFilter = filter; // Store filter selection
        let filteredUsers = this.originalUsersDataSource.data;

        if (filter === 'banned') {
            filteredUsers = filteredUsers.filter(user => user.banned);
        } else if (filter === 'notBanned') {
            filteredUsers = filteredUsers.filter(user => !user.banned);
        }

        this.filteredUsersDataSource.data = [...filteredUsers];
    }
}
